# AMICI documentation

This file describes how the AMICI documentation is organized and compiled.

The AMICI documentation is created using [doxygen](http://www.doxygen.nl/).
It combines Markdown files from the root directory, from `documentation/`
and in-source documentation from the C++, Python and Matlab source files.

The documentation is generated by running

    scripts/run-doxygen.sh

The resulting HTML and PDF documentation will be created in `doc/`. Upon
merges to the `master` branch, the documentation will be deployed to
github-pages at http://icb-dcm.github.io/AMICI/.

`scripts/run-doxygen.sh` also checks for any missing in-source documentation.


## Doxygen configuration

The main doxygen configuration file is located in
`matlab/mtoc/config/Doxyfile.template`. Edit this file for inclusion or
exclusion of additional files.


## Matlab documentation

Matlab documentation is processed by
[mtoc++](https://www.morepas.org/software/mtocpp/docs/tools.html).
This is configured in `matlab/mtoc/config`.


## Python documentation

Python documentation is processed by doxygen and doxypypy using the script and
filters in `scripts/`.


## Out-of-source documentation

Out-of-source documentation files should be written in Markdown and
created within `documentation/`.
Graphics for documentation are kept in `documentation/gfx/`.

Some guidelines:

* Note that there are some incompatibilities of Github Markdown and Doxygen
  Markdown. Ideally documentation should be written in a format compatible with
  both.
  This affects for example images links which currently cause trouble in
  Doxygen.

* Where possible, relative links are preferred over absolute links. However,
  they should work with both Github and Doxygen and ideally with local files
  for offline use.

* Please stick to the limit of 80 characters per line for readability of raw
  Markdown files where possible. 

  However, note that some Markdown interpreters can handle line breaks within
  links and headings, whereas others cannot. Here, compatibility is preferred
  over linebreaks. 
  
* Avoid trailing whitespace


## Maintaining the list of publications

We want to maintain a list of publications / projects using AMICI. This is
located at `documentation/references.md`. This file is created by
`documentation/recreate_reference_list.py` based on
the bibtex file `documentation/amici_refs.bib`.

After any changes to `documentation/amici_refs.bib`, please run

    documentation/recreate_reference_list.py
